# Dropzone Action Info
# Name: Imgur
# Description: Uploads images to Imgur.\n\nDirect link to the image will be put on clipboard when upload completes. Hold option while dragging to get link to image on Imgur rather than the direct link.\n\nDrop multiple images to create an album.
# Handles: Files
# Creator: Aptonic Software
# URL: http://aptonic.com
# OptionsNIB: Imgur
# Events: Clicked, Dragged
# KeyModifiers: Option
# SkipConfig: No
# RunsSandboxed: Yes
# Version: 1.0
# MinDropzoneVersion: 3.0

require 'imgur'
require 'curl_uploader'

def dragged
  imgur = Imgur.new
  
  $dz.determinate(false)
  
  allowed_exts = ["jpg", "jpeg", "gif", "tif", "tiff", "png", "bmp"]
  
  # Check only supported types were dragged
  $items.each do |item|
    filename = File.basename(item)
    ext = File.extname(item).downcase[1..-1]  
    $dz.fail("Only image files are supported") if not allowed_exts.include?(ext)
  end

  album_name = $dz.inputbox("Upload #{$items.length} Images to Imgur", "Enter album name:", "Album name") if $items.length > 1

  uploader = CurlUploader.new
  uploader.upload_url = "https://api.imgur.com/3/image"
  uploader.file_field_name = "image"
  
  access_token = nil
  
  if ENV["access_token"]
    # Renew the access token and set Authorization: Bearer header accordingly
    access_token = imgur.renew_auth_token(ENV['client_id'], ENV['client_secret'], ENV['refresh_token'])
    
    unless access_token
      $dz.error("Failed to renew auth token", "Try removing and readding your Imgur account.\n\nDebug Info:: #{imgur.error_message}")
    end
    
    uploader.headers["Authorization"] = "Bearer #{access_token}"
  else
    uploader.headers["Authorization"] = "Client-ID #{ENV['client_id']}"
  end
  
  if $items.length == 1
    # Just do the upload and put URL on clipboard
    result = uploader.upload($items)[0]
    puts result.inspect
    check_upload_output_valid(result)
    if ENV["KEY_MODIFIERS"] == "Option"
      finish("https://imgur.com/" + result[:output]["data"]["id"])
    else
      finish(result[:output]["data"]["link"])
    end
  else
    # Create album, upload to it then put album URL on clipboard
    album_info = imgur.create_album(album_name, access_token)

    if access_token
      uploader.post_vars["album"] = album_info["id"]
    else
      uploader.post_vars["album"] = album_info["deletehash"]
    end
      
    results = uploader.upload($items)
    results.each do |result|
      check_upload_output_valid(result)
    end
    
    finish("https://imgur.com/a/#{album_info["id"]}", "Album:: #{album_name}")
  end
end

def check_upload_output_valid(result)
  json_result = result[:output]
  if result[:curl_output_valid] and json_result["success"] and json_result["data"]
    return true
  else
    error_message = json_result
    if json_result["data"] and json_result["data"]["error"]
      error_message = json_result["data"]["error"]
    end
    $dz.fail(error_message)
  end
end

def finish(url, title=nil)
  $dz.finish("URL is now on clipboard")
  $dz.url(url, title)
end

def clicked
  if ENV["access_token"]
    system("open https://#{ENV['account_username']}.imgur.com/all/")
  else
    system("open https://imgur.com")
  end
end
