require 'post_request'
require 'json'

class Imgur
  attr_accessor :error_message
  
  # Methods to refresh_token and create_album go here
  def renew_auth_token(client_id, client_secret, refresh_token)
    
    form_data = {
    	"client_id" => client_id,
    	"client_secret" => client_secret,
    	"refresh_token" => refresh_token,
    	"grant_type" => "refresh_token"
    }
    
    post_request = PostRequest.new("https://api.imgur.com/oauth2/token", form_data)

    begin
      response = post_request.post
      json = JSON.parse(response)
    rescue
      @error_message = $!
      return false
    end

    if json["access_token"] and json["refresh_token"]
      $dz.save_value("access_token", json["access_token"])
      $dz.save_value("refresh_token", json["refresh_token"])
      return json["access_token"]
    else
      @error_message = "Invalid JSON received: #{json.inspect}"
      return false
    end
  end
  
  def create_album(album_name, access_token=nil)
    form_data = {
    	"title" => album_name
    }
    
    if (access_token)
      auth_header = {"Authorization" => "Bearer #{access_token}"}
    else
      auth_header = {"Authorization" => "Client-ID #{ENV['client_id']}"}
    end
    
    post_request = PostRequest.new("https://api.imgur.com/3/album", form_data, auth_header)

    album_creation_success = false

    begin
      response = post_request.post
      json = JSON.parse(response)
      if json["success"] and json["data"]
        album_creation_success = true
      else
        album_creation_success = false
      end
    rescue
      album_creation_success = false
      error_message = $!
    end
    
    if (album_creation_success)
      return json["data"]
    else
      error_output = (error_message ? "Error:: #{error_message}\n\nResponse: #{response}" : "Response not valid:: #{response}")
      $dz.error("Album creation failed", "The album could not be created.\n\n#{error_output}")
    end
    
  end
  
end