# Dropzone Action Info
# Name: SFTP Upload
# Description: Allows files to be uploaded to a remote SFTP server. If the option key is held down then files are zipped up before uploading.
# Handles: Files
# Creator: Aptonic Software
# URL: https://aptonic.com
# OptionsNIB: ExtendedLogin
# DefaultPort: 22
# Events: Dragged, TestConnection
# KeyModifiers: Option
# SkipConfig: No
# RunsSandboxed: Yes
# Version: 1.4
# MinDropzoneVersion: 3.0
# ForceX86Ruby: Yes

require 'sftp_uploader'

$host_info = {:server    => ENV['server'],
              :port      => ENV['port'],
              :username  => ENV['username'],
              :password  => ENV['password']}

def dragged
  delete_zip = false

  if ENV['KEY_MODIFIERS'] == "Option"
    # Zip up files before uploading
    if $items.length == 1
      # Use directory or file name as zip file name
      dir_name = $items[0].split(File::SEPARATOR)[-1]
      file = ZipFiles.zip($items, "#{dir_name}.zip")
    else
      file = ZipFiles.zip($items, "files.zip")
    end
    
    # Remove quotes
    items = [file[1..-2]]
    delete_zip = true
  else
    # Recursive upload      
    items = $items
  end
  
  $dz.begin("Starting transfer...")
  $dz.determinate(false)

  remote_paths = SFTPUploader.do_upload(items, ENV['remote_path'], $host_info)
  ZipFiles.delete_zip(items[0]) if delete_zip
  
  # Put URL of uploaded file on pasteboard
  finish_text = "Upload Complete"
  
  if remote_paths.length == 1
    filename = remote_paths[0].split(File::SEPARATOR)[-1].strip
    
    if ENV['root_url'] != nil
      slash = (ENV['root_url'][-1,1] == "/" ? "" : "/")
      url = ENV['root_url'] + slash + filename
      finish_text = "URL is now on clipboard"
    else
      url = filename
    end
  else
    url = false
  end
  
  $dz.finish(finish_text)
  $dz.url(url)
end

def test_connection
  SFTPUploader.test_connection($host_info)
end
